package com.lqkj.link.module.authority.service;

import com.lqkj.link.module.authority.domain.RoleInfo;
import com.lqkj.link.module.authority.repository.RoleInfoRepository;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

@Service
public class RoleInfoService {
    private final RoleInfoRepository roleInfoRepository;

    public RoleInfoService(RoleInfoRepository roleInfoRepository) {
        this.roleInfoRepository = roleInfoRepository;
    }

    public List<RoleInfo> allRoles() {
        return roleInfoRepository.queryAll();
    }

    public Page<RoleInfo> pageQuery(String roleName, Integer page, Integer pageSize) {
        Pageable pageable = PageRequest.of(page, pageSize);
        Page<RoleInfo> roleInfoPage = roleInfoRepository.pageQueryV2(roleName, pageable);
        for (RoleInfo role : roleInfoPage) {
            role.setAuths(roleInfoRepository.queryAuthNamesWithRole(role.getRoleId()));
        }
        return roleInfoPage;
    }

    public RoleInfo detail(Integer roleId) {
        RoleInfo roleInfo = roleInfoRepository.findById(roleId).get();
        List<Integer> auths = roleInfoRepository
                .queryAuthsWithRole(roleId)
                .stream()
                .map(v -> (Integer) v.get("authority_id"))
                .toList();
        roleInfo.setAuthorityList(auths);
        return roleInfo;
    }

    @Transactional
    public String save(RoleInfo roleInfo) {
        List<Integer> authorityIds = roleInfo.getAuthorityList();
        if (roleInfo.getRoleId() == null) {
            if (roleInfoRepository.hasSameEnName(roleInfo.getEnName())) {
                return "英文名重复！";
            }
            if (roleInfoRepository.hasSameName(roleInfo.getRoleName())) {
                return "角色名称重复！";
            }
            roleInfo.setUpdateTime(new Date());
            roleInfo = roleInfoRepository.save(roleInfo);
        } else {
            if (roleInfoRepository.hasSameEnNameWithoutOneId(roleInfo.getEnName(), roleInfo.getRoleId())) {
                return "英文名重复！";
            }
            if (roleInfoRepository.hasSameNameWithoutOneId(roleInfo.getRoleName(), roleInfo.getRoleId())) {
                return "角色名称重复！";
            }
            RoleInfo oldRole = roleInfoRepository.findById(roleInfo.getRoleId()).get();
            oldRole.setRoleName(roleInfo.getRoleName());
            oldRole.setEnName(roleInfo.getEnName());
            oldRole.setUpdateTime(new Date());
            roleInfoRepository.save(oldRole);
        }

        roleInfoRepository.updateRoleAuthority(roleInfo.getRoleId(), authorityIds);
        return null;
    }

    public List<Map<String, Object>> queryAuthTree(String userCode) {
        List<Map<String, Object>> resultList = new ArrayList<>();
        Map<Integer, List<Map<String, Object>>> childrenMap = new HashMap<>();

        List<Map<String, Object>> authList = roleInfoRepository.queryAuthsByUser(userCode);
        for (Map<String, Object> auth : authList) {
            Map<String, Object> result = new HashMap<>();
            result.put("id", auth.get("authority_id"));
            result.put("name", auth.get("authority_name"));

            Integer parentId = (Integer) auth.get("parent_id");
            if (parentId == null) {
                result.put("children", childrenMap.get((Integer) auth.get("authority_id")));
                resultList.add(result);
            } else {
                List<Map<String, Object>> children = childrenMap.getOrDefault(parentId, new ArrayList<>());
                children.add(result);
                childrenMap.put(parentId, children);
            }
        }

        return resultList;
    }

    public String delete(List<Integer> roleIds) {
        return roleInfoRepository.deleteNoUserRole(roleIds);
    }
}
