package com.lqkj.link.module.bulletin.repository;

import com.lqkj.link.module.bulletin.domain.NoticeInfo;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;
import org.springframework.stereotype.Repository;

import java.util.List;

@Repository
public interface NoticeInfoRepository extends JpaRepository<NoticeInfo, Integer> {

    @Modifying
    @Query(nativeQuery = true,
        value = "insert into notice_info(info_id, bulletin_id, content, create_time, user_id, has_view, type) " +
                "select nextval('notice_info_info_id_seq'), :bulletinId, bi.content, bi.create_time, ui.user_id, false, bi.info_type " +
                "from bulletin_info bi, user_info ui " +
                "where bi.info_id = :bulletinId"
    )
    void addBulletin(@Param("bulletinId") Integer bulletinId);

    @Modifying
    @Query(nativeQuery = true,
        value = "update notice_info set content = :content, create_time = now(), has_view = false, type = :type " +
                "where bulletin_id = :bulletinId"
    )
    void updateBulletin(@Param("bulletinId") Integer bulletinId,
                        @Param("content") String content,
                        @Param("type") Integer type);

    @Modifying
    @Query(nativeQuery = true,
        value = "delete from notice_info where bulletin_id = :bulletinId"
    )
    void deleteBulletin(@Param("bulletinId") Integer bulletinId);

    @Query(nativeQuery = true,
        value = "select ni.* from notice_info ni, user_info ui " +
                "where ni.user_id = ui.user_id and ui.user_code = :userCode " +
                "order by create_time desc"
    )
    List<NoticeInfo> queryWithUserCode(@Param("userCode") String userCode);

    @Modifying
    @Query(nativeQuery = true,
        value = "insert into notice_info(info_id, bulletin_id, content, create_time, user_id, has_view, view_time, type) " +
                "select nextval('notice_info_info_id_seq'), null, :content, now(), :userId, false, null, 3 "
    )
    void createAuditNotice(@Param("content") String content,
                           @Param("userId") Integer userId);

    @Modifying
    @Query(nativeQuery = true,
        value = "update notice_info set has_view = true where info_id = :infoId"
    )
    void view(@Param("infoId") Integer infoId);

}
