package com.lqkj.link.module.bulletin.service;

import com.lqkj.link.module.bulletin.domain.BulletinInfo;
import com.lqkj.link.module.bulletin.repository.BulletinInfoRepository;
import com.lqkj.link.module.bulletin.repository.NoticeInfoRepository;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.data.domain.Sort;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

@Service
public class BulletinInfoService {
    private final BulletinInfoRepository bulletinInfoRepository;
    private final NoticeInfoRepository noticeInfoRepository;

    public BulletinInfoService(BulletinInfoRepository bulletinInfoRepository, NoticeInfoRepository noticeInfoRepository) {
        this.bulletinInfoRepository = bulletinInfoRepository;
        this.noticeInfoRepository = noticeInfoRepository;
    }

    public List<BulletinInfo> queryAll() {
        return bulletinInfoRepository.findAll(Sort.by(Sort.Direction.DESC, "createTime"));
    }

    @Transactional
    public void save(BulletinInfo bulletinInfo) {
        if (bulletinInfo.getInfoId() == null) {
            bulletinInfo.setCreateTime(new Date());
            BulletinInfo newInfo = bulletinInfoRepository.save(bulletinInfo);
            noticeInfoRepository.addBulletin(newInfo.getInfoId());
        } else {
            bulletinInfo.setCreateTime(new Date());
            bulletinInfoRepository.save(bulletinInfo);
            noticeInfoRepository.updateBulletin(bulletinInfo.getInfoId(), bulletinInfo.getContent(), bulletinInfo.getInfoType());
        }
    }

    @Transactional
    public void delete(Integer infoId) {
        bulletinInfoRepository.deleteById(infoId);
        noticeInfoRepository.deleteBulletin(infoId);
    }
}
