package com.lqkj.link.module.config.service;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.lqkj.link.module.config.domain.ConfigInfo;
import com.lqkj.link.module.config.repository.ConfigInfoRepository;
import jakarta.annotation.PostConstruct;
import org.springframework.data.domain.Sort;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
public class ConfigInfoService {
    private final ConfigInfoRepository configInfoRepository;

    public ConfigInfoService(ConfigInfoRepository configInfoRepository) {
        this.configInfoRepository = configInfoRepository;
    }

    public List<ConfigInfo> queryAll() {
        return configInfoRepository.findAll(Sort.by(Sort.Direction.ASC, "configKey"));
    }

    public void saveAll(List<ConfigInfo> configInfoList) {
        configInfoRepository.saveAll(configInfoList);
    }

    public ConfigInfo getOne(String configKey) {
        return configInfoRepository.findById(configKey).get();
    }

    @PostConstruct
    public void saveVersion() {
        ConfigInfo configInfo;
        if (configInfoRepository.existsById("jarVersion")) {
            configInfo = configInfoRepository.findById("jarVersion").get();
            String configValue = configInfo.getConfigValue();
            JSONObject jsonObject = JSON.parseObject(configValue);
            jsonObject.put("java", "V1.0.0");
            configInfo.setConfigValue(jsonObject.toJSONString());
            configInfo.setContentType("application/json");
        } else {
            JSONObject jsonObject = new JSONObject();
            jsonObject.put("java", "V1.0.0");
            configInfo = new ConfigInfo("jarVersion", jsonObject.toJSONString(), "application/json");
        }
        configInfoRepository.save(configInfo);
    }
}
